
/**************************************************************************
**  (c) Copyright 2002, Andromeda Technology & Automation
** This is free software; you can redistribute it and/or modify it under the
** terms of the GNU General Public License, see the file COPYING.
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : irix_syslog_cooker.cpp
**      SYSTEM NAME    : 
**      VERSION NUMBER : $Revision: 1.4 $
**
**  DESCRIPTION      :  Cooks IRIX system log lines
**
**  EXPORTED OBJECTS : 
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Aug 09, 2003
**      LAST UPDATE     : Aug 09, 2003
**      MODIFICATIONS   : 
**************************************************************************/

/*****************************
   $Log: irix_syslog_cooker.cpp,v $
   Revision 1.4  2007-10-19 07:22:12  arjen
   When extracting the service name for a log entry, the '/' character
   is not considered part of that name.

   Revision 1.3  2007/01/11 13:51:40  arjen
   The '-' character is part of the service name.

   Revision 1.2  2004/01/13 12:20:13  arjen
   Accept '/' and '.' characters within the service name

   Revision 1.1  2003/08/11 16:56:16  arjen
   Different kinds of log files are parsed by a collection of objects
   of different classes, derived from the base class line_cooker
   Depending on the message content or the message_type element in
   XML, one of these objects is selected.

   Logrunner is integrated with gcm_input. Although its functionality
   is still limited, a connection between logrunner and gcm_input
   is beginning to form.

*****************************/

/* static const char *RCSID = "$Id: irix_syslog_cooker.cpp,v 1.4 2007-10-19 07:22:12 arjen Exp $"; */

#include <ctype.h>

#include "irix_syslog_cooker.h"

static const String syslog_date_re("[[:alpha:]]{3} [ 123][0-9] [0-9]{2}:[0-9]{2}:[0-9]{2}");
static const regex re_syslog_irix(syslog_date_re + " [0-7][A-T]:[[:alnum:]]+ [[:alpha:]/]+.*:.+");

bool irix_syslog_cooker::check_pattern(String logline)
{
   return logline == re_syslog_irix;
}

bool irix_syslog_cooker::cook_this(String logline, UTC arrival)
{
   if (check_pattern(logline))
   {
      String datestring = logline(0,16);
      date   log_date = datestring;
      hour   log_time = datestring;
      String rest;   //  Rest of the line to be parsed
      int    i;

      if (log_date.Year() < 0 || log_date.Year() > 2500)
      {
         //  The year is not in the log file. Assume the year of arrival,
         //  unless this puts the log entry at a later date than the arrival date.
         //  This happens e.g. when a log entry from December arrives in Januari.

         log_date = date(log_date.Day(), log_date.Month(), date(arrival).Year());
         if (log_date > date(arrival))
         {
            log_date = date(log_date.Day(), log_date.Month(), date(arrival).Year() - 1);
         }
      }

      ts = UTC(log_date, log_time);

      //  Extract the hostname
 
      rest = logline << 19;
      i = rest.index(' ');
      hn = rest(0,i);

      //  Extract the service

      rest <<= i + 1;
      for (i = 0; (isalpha(rest[i])  || rest[i] == '.' || rest[i] == '-') && i < ~rest; i++);
      srv = rest(0, i);

      return true;
   }
   else
   {
      return false;
   }
}


